USE sgvault
GO

-- this user function will retrieve correct rights for a given user.
IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id(N'[dbo].[ufngetusersecurityrights]') AND xtype IN (N'FN', N'IF', N'TF'))
	DROP FUNCTION [dbo].[ufngetusersecurityrights]
GO
CREATE FUNCTION [dbo].[ufngetusersecurityrights] ( @repid [int], @userid [int], @txid [bigint] = -1 )
RETURNS @securityrights TABLE 
(	ident [int] IDENTITY(1,1) NOT NULL,
	type [nchar](1) COLLATE DATABASE_DEFAULT NOT NULL, -- default, user, or group
	objid [bigint] NOT NULL, 
	usergroupid [int] NOT NULL, 
	fullpath [nvarchar](1024) COLLATE DATABASE_DEFAULT NOT NULL,  -- ufullpath
	securityrights [int] NOT NULL,  -- usecurityrights
	PRIMARY KEY (type, objid, usergroupid, ident)
) WITH ENCRYPTION
AS 
BEGIN

--          Copyright 2002-2005 SourceGear LLC.                    --
--          All rights reserved.  See Vault License Agreement      --
--          for more information.                                  --

-- PLEASE NOTE - this ufn must never return the same objectid but w/ different security
-- rights.  if this occurs, it means something has been incorrectly set in tblsecurityassignments
-- and the other parts of the schema which depend on only one objid / security pairing will fail.

DECLARE @@repidparam [int], 
	@@useridparam [int], 
	@@txidparam [bigint]

SELECT @@repidparam = @repid, 
	@@useridparam = @userid, 
	@@txidparam = COALESCE(@txid, -1)

-- default rights
INSERT INTO @securityrights (type, objid, usergroupid, fullpath, securityrights) 
SELECT N'D', 0, @@useridparam, N'!', securityrights FROM dbo.tblusers WHERE userid = @@useridparam

IF (@@txidparam = -1)
BEGIN
	-- when txid = -1, just get the current rights.
	INSERT INTO @securityrights (type, objid, usergroupid, fullpath, securityrights) 
	(
		-- get all group assignments  note the most permissive statements have higher securityright values
		-- be sure to exclude any conflicting group rights.
		SELECT N'G', r.objid, r.groupid, sap.fullpath, r.securityrights FROM dbo.tblsecurityassignments r 
			INNER JOIN dbo.tblsecurityassignmentspaths sap ON ((sap.repid = r.repid) AND (sap.objid = r.objid) AND (sap.endtxid IS NULL))
			INNER JOIN dbo.tblfsobjects o ON ((o.repid=@@repidparam) AND (o.objid = r.objid)) 
			INNER JOIN dbo.tblgroupmembers gm ON ((gm.groupid=r.groupid) AND (gm.userid = @@useridparam)) 
			WHERE (r.repid = @@repidparam) 
			-- not already assigned to a user.
			AND ( NOT EXISTS (SELECT r1.objid FROM dbo.tblsecurityassignments r1 WHERE ((r1.objid=r.objid) AND (r1.userid=@@useridparam) AND (r1.objid=r.objid)) ) ) 
			-- select the largest security right joined to itself to resolve equivalent path assignment conflicts
			AND ( (r.securityrights >= 
			  (SELECT MAX(r2.securityrights) FROM dbo.tblsecurityassignments r2 
				  INNER JOIN dbo.tblsecurityassignmentspaths sap2 ON ( (sap2.repid = r2.repid) AND (sap2.objid = r2.objid) AND (sap2.endtxid IS NULL) ) 
				  INNER JOIN dbo.tblgroupmembers gm2 ON ((gm2.groupid=r2.groupid) AND (gm2.userid=@@useridparam)) 
				  WHERE (r2.objid=r.objid) AND (r2.userid=r.userid)) ) 
			)
			-- do not retrieve GROUP rights where this USER right is assigned from above.
			AND ( NOT EXISTS (SELECT sap3.objid FROM dbo.tblsecurityassignments sa3 
						INNER JOIN dbo.tblsecurityassignmentspaths sap3 ON (sap3.repid = sa3.repid) AND (sap3.objid = sa3.objid) AND (sap3.endtxid IS NULL) 
						WHERE (sa3.repid = sap.repid) AND (sa3.userid = @@useridparam) AND (sap.fullpath LIKE sap3.fullpath + N'/%')) )
	)
	UNION ALL 
	( 
		-- override group rights w/ user rights
		SELECT N'U', r.objid, r.userid, sap.fullpath, r.securityrights FROM dbo.tblsecurityassignments r 
			INNER JOIN dbo.tblsecurityassignmentspaths sap ON (sap.repid = r.repid) AND (sap.objid = r.objid) AND (sap.endtxid IS NULL)
			WHERE r.repid=@@repidparam AND r.userid = @@useridparam
	)
END
ELSE
BEGIN
	-- txid <> -1, get all rights where the txid is in between start / end txid
	INSERT INTO @securityrights (type, objid, usergroupid, fullpath, securityrights) 
	(
		-- get all group assignments  note the most permissive statements have higher securityright values
		-- be sure to exclude any conflicting group rights.
		SELECT DISTINCT N'G', r.objid, r.groupid, sap.fullpath, r.securityrights FROM dbo.tblsecurityassignments r 
			INNER JOIN dbo.tblsecurityassignmentspaths sap ON (sap.repid = r.repid) AND (sap.objid = r.objid) AND (@@txidparam BETWEEN sap.begintxid AND COALESCE(sap.endtxid, 9223372036854775807))
			INNER JOIN dbo.tblfsobjects o ON ((o.repid=@@repidparam) AND (o.objid = r.objid)) 
			INNER JOIN dbo.tblgroupmembers gm ON ((gm.groupid=r.groupid) AND (gm.userid = @@useridparam)) 
			WHERE (r.repid = @@repidparam) 
			-- not already assigned to a user.
			AND ( NOT EXISTS (SELECT r1.objid FROM dbo.tblsecurityassignments r1 WHERE ((r1.objid=r.objid) AND (r1.userid=@@useridparam) AND (r1.objid=r.objid)) ) ) 
			-- select the largest security right joined to itself to resolve equivalent path assignment conflicts
			AND ( (r.securityrights >= 
			  (SELECT MAX(r2.securityrights) FROM dbo.tblsecurityassignments r2 
				  INNER JOIN dbo.tblsecurityassignmentspaths sap2 ON ( (sap2.repid = r2.repid) AND (sap2.objid = r2.objid) AND (@@txidparam BETWEEN sap2.begintxid AND COALESCE(sap2.endtxid, 9223372036854775807)) ) 
				  INNER JOIN dbo.tblgroupmembers gm2 ON ((gm2.groupid=r2.groupid) AND (gm2.userid=@@useridparam)) 
				  WHERE (r2.objid=r.objid) AND (r2.userid=r.userid)) ) 
			)
			-- do not retrieve GROUP rights where this USER right is assigned from above.
			AND ( NOT EXISTS (SELECT sap3.objid FROM dbo.tblsecurityassignments sa3 
				INNER JOIN dbo.tblsecurityassignmentspaths sap3 ON (sap3.repid = sa3.repid) AND (sap3.objid = sa3.objid) AND (@@txidparam BETWEEN sap3.begintxid AND COALESCE(sap3.endtxid, 9223372036854775807))
				WHERE (sa3.repid = sap.repid) AND (sa3.userid = @@useridparam) AND (sap.fullpath LIKE sap3.fullpath + N'/%')))
	)
	UNION ALL 
	( 
		-- override group rights w/ user rights
		SELECT N'U', r.objid, r.userid, sap.fullpath, r.securityrights FROM dbo.tblsecurityassignments r 
			INNER JOIN dbo.tblsecurityassignmentspaths sap ON (sap.repid = r.repid) AND (sap.objid = r.objid) AND (@@txidparam BETWEEN sap.begintxid AND COALESCE(sap.endtxid, 9223372036854775807))
			WHERE r.repid=@@repidparam AND r.userid = @@useridparam
	)
END

RETURN
END
GO
